@@ -2,6 +2,6 @@ import darwin.dataset  # noqa
 import darwin.exceptions  # noqa
 
 from .client import Client  # noqa
-from .team import Team  # noqa
+from .datatypes import Team  # noqa
 
-__version__ = "0.6.14"
+__version__ = "0.6.15"
