@@ -1,11 +1,13 @@
 from __future__ import annotations
 
-from typing import Callable, List, Optional
+from typing import Any, Callable, Dict, List, Optional, Tuple
 
 import numpy as np
+import PIL
 from darwin.cli_functions import _error, _load_client
 from darwin.dataset import LocalDataset
 from darwin.dataset.identifier import DatasetIdentifier
+from darwin.datatypes import Annotation
 from darwin.torch.transforms import (
     Compose,
     ConvertPolygonsToInstanceMasks,
@@ -13,6 +15,8 @@ from darwin.torch.transforms import (
 )
 from darwin.torch.utils import polygon_area
 from darwin.utils import convert_polygons_to_sequences
+from PIL import Image as PILImage
+from torchvision.transforms.functional import to_tensor
 
 import torch
 
@@ -24,7 +28,7 @@ def get_dataset(
     split: str = "default",
     split_type: str = "random",
     transform: Optional[List] = None,
-):
+) -> LocalDataset:
     """
     Creates and returns a dataset
 
@@ -81,14 +85,15 @@ class ClassificationDataset(LocalDataset):
         """
         super().__init__(annotation_type="tag", **kwargs)
 
-        self.transform = transform
-        if self.transform is not None and isinstance(self.transform, list):
-            self.transform = Compose(self.transform)
+        if transform is not None and isinstance(transform, list):
+            transform = Compose(transform)
+
+        self.transform: Optional[Callable] = transform
 
         self.is_multi_label = False
         self.check_if_multi_label()
 
-    def __getitem__(self, index: int):
+    def __getitem__(self, index: int) -> Tuple[torch.Tensor, torch.Tensor]:
         """
         See superclass for documentation
 
@@ -102,15 +107,17 @@ class ClassificationDataset(LocalDataset):
             category_id : int
                 The single label of the image selected
         """
-        img = self.get_image(index)
+        img: PILImage.Image = self.get_image(index)
         if self.transform is not None:
-            img = self.transform(img)
+            img_tensor = self.transform(img)
+        else:
+            img_tensor = to_tensor(img)
 
         target = self.get_target(index)
 
-        return img, target
+        return img_tensor, target
 
-    def get_target(self, index: int):
+    def get_target(self, index: int) -> torch.Tensor:
         """
         Returns the classification target
         """
@@ -121,7 +128,7 @@ class ClassificationDataset(LocalDataset):
 
         assert len(tags) >= 1, f"No tags were found for index={index}"
 
-        target = torch.tensor(self.classes.index(tags[0]))
+        target: torch.Tensor = torch.tensor(self.classes.index(tags[0]))
 
         if self.is_multi_label:
             target = torch.zeros(len(self.classes))
@@ -145,8 +152,8 @@ class ClassificationDataset(LocalDataset):
                 self.is_multi_label = True
                 break
 
-    def get_class_idx(self, index: int):
-        target = self.get_target(index)
+    def get_class_idx(self, index: int) -> int:
+        target: torch.Tensor = self.get_target(index)
         return target["category_id"]
 
     def measure_weights(self, **kwargs) -> np.ndarray:
@@ -169,19 +176,20 @@ class ClassificationDataset(LocalDataset):
 
 
 class InstanceSegmentationDataset(LocalDataset):
-    def __init__(self, transform: Optional[List] = None, **kwargs):
+    def __init__(self, transform: Optional[Callable | List] = None, **kwargs):
         """
         See `LocalDataset` class for documentation
         """
         super().__init__(annotation_type="polygon", **kwargs)
 
-        self.transform = transform
-        if self.transform is not None and isinstance(self.transform, list):
-            self.transform = Compose(self.transform)
+        if transform is not None and isinstance(transform, list):
+            transform = Compose(transform)
+
+        self.transform: Optional[Callable] = transform
 
         self.convert_polygons = ConvertPolygonsToInstanceMasks()
 
-    def __getitem__(self, index: int):
+    def __getitem__(self, index: int) -> Tuple[torch.Tensor, Dict[str, Any]]:
         """
         Notes
         -----
@@ -199,16 +207,18 @@ class InstanceSegmentationDataset(LocalDataset):
             area : float
                 Area in pixels of each one of the instances
         """
-        img = self.get_image(index)
-        target = self.get_target(index)
+        img: PILImage.Image = self.get_image(index)
+        target: Dict[str, Any] = self.get_target(index)
 
         img, target = self.convert_polygons(img, target)
         if self.transform is not None:
-            img, target = self.transform(img, target)
+            img_tensor, target = self.transform(img, target)
+        else:
+            img_tensor = to_tensor(img)
 
-        return img, target
+        return img_tensor, target
 
-    def get_target(self, index: int):
+    def get_target(self, index: int) -> Dict[str, Any]:
         """
         Returns the instance segmentation target
         """
@@ -221,9 +231,7 @@ class InstanceSegmentationDataset(LocalDataset):
             # Extract the sequences of coordinates from the polygon annotation
             annotation_type: str = "polygon" if "polygon" in annotation else "complex_polygon"
             sequences = convert_polygons_to_sequences(
-                annotation[annotation_type]["path"],
-                height=target["height"],
-                width=target["width"],
+                annotation[annotation_type]["path"], height=target["height"], width=target["width"],
             )
             # Compute the bbox of the polygon
             x_coords = [s[0::2] for s in sequences]
@@ -251,7 +259,7 @@ class InstanceSegmentationDataset(LocalDataset):
 
         return target
 
-    def measure_weights(self, **kwargs):
+    def measure_weights(self, **kwargs) -> np.ndarray:
         """
         Computes the class balancing weights (not the frequencies!!) given the train loader
         Get the weights proportional to the inverse of their class frequencies.
@@ -263,7 +271,7 @@ class InstanceSegmentationDataset(LocalDataset):
             Weight for each class in the train set (one for each class) as a 1D array normalized
         """
         # Collect all the labels by iterating over the whole dataset
-        labels = []
+        labels: List[int] = []
         for i, _ in enumerate(self.images_path):
             target = self.get_target(i)
             labels.extend([a["category_id"] for a in target["annotations"]])
@@ -271,19 +279,19 @@ class InstanceSegmentationDataset(LocalDataset):
 
 
 class SemanticSegmentationDataset(LocalDataset):
-    def __init__(self, transform: Optional[List] = None, **kwargs):
+    def __init__(self, transform: Optional[List | Callable] = None, **kwargs):
         """
         See `LocalDataset` class for documentation
         """
         super().__init__(annotation_type="polygon", **kwargs)
 
-        self.transform = transform
-        if self.transform is not None and isinstance(self.transform, list):
-            self.transform = Compose(self.transform)
+        if transform is not None and isinstance(transform, list):
+            transform = Compose(transform)
 
+        self.transform: Optional[Callable] = transform
         self.convert_polygons = ConvertPolygonsToSemanticMask()
 
-    def __getitem__(self, index: int):
+    def __getitem__(self, index: int) -> Tuple[torch.Tensor, Dict[str, Any]]:
         """
         See superclass for documentation
 
@@ -297,16 +305,18 @@ class SemanticSegmentationDataset(LocalDataset):
             mask : tensor(H, W)
                 Segmentation mask where each pixel encodes a class label
         """
-        img = self.get_image(index)
-        target = self.get_target(index)
+        img: PILImage.Image = self.get_image(index)
+        target: Dict[str, Any] = self.get_target(index)
 
         img, target = self.convert_polygons(img, target)
         if self.transform is not None:
-            img, target = self.transform(img, target)
+            img_tensor, target = self.transform(img, target)
+        else:
+            img_tensor = to_tensor(img)
 
-        return img, target
+        return img_tensor, target
 
-    def get_target(self, index: int):
+    def get_target(self, index: int) -> Dict[str, Any]:
         """
         Returns the semantic segmentation target
         """
@@ -315,9 +325,7 @@ class SemanticSegmentationDataset(LocalDataset):
         annotations = []
         for obj in target["annotations"]:
             sequences = convert_polygons_to_sequences(
-                obj["polygon"]["path"],
-                height=target["height"],
-                width=target["width"],
+                obj["polygon"]["path"], height=target["height"], width=target["width"],
             )
             # Discard polygons with less than three points
             sequences[:] = [s for s in sequences if len(s) >= 6]
@@ -328,7 +336,7 @@ class SemanticSegmentationDataset(LocalDataset):
 
         return target
 
-    def measure_weights(self, **kwargs):
+    def measure_weights(self, **kwargs) -> np.ndarray:
         """
         Computes the class balancing weights (not the frequencies!!) given the train loader
         Get the weights proportional to the inverse of their class frequencies.
